@@ -117,10 +117,10 @@ about_callback (GtkWidget *widget,
       "Damien Sandras <dsandras@seconix.com>",
       "",
       N_("Code contributors:"),
-      "Benjamin Leviant <belevian@gmail.com>"
+      "Benjamin Leviant <belevian@gmail.com>",
       "Kenneth Rohde Christiansen <kenneth@gnu.org>",
       "Julien Puydt <julien.puydt@laposte.net>",
-      "Miguel Rodríguez Pérez <migrax@terra.es>",
+      "Miguel Rodríguez Pérez <miguelrp@gmail.com>",
       "Paul <paul@argo.dyndns.org>", 
       "Roger Hardiman <roger@freebsd.org>",
       "Sébastien Josset <Sebastien.Josset@space.alcatel.fr>",
