@@ -542,6 +542,7 @@ handle_connect_to_message (DBusConnection *connection,
 			     DBUS_TYPE_INVALID)) {
 
     GnomeMeeting::Process ()->Connect (address);
+    g_free (address);
   }
 }
 
@@ -580,6 +581,7 @@ handle_disconnect_message (DBusConnection *connection,
   
     /* FIXME: should use call_token, when gnomemeeting will support it! */
     GnomeMeeting::Process ()->Disconnect ();
+    g_free (call_token);
   }
 }
 
@@ -642,6 +644,9 @@ handle_get_call_info_message (DBusConnection *connection,
       dbus_connection_flush (connection);
     }
     dbus_message_unref (reply);
+    g_free (name);
+    g_free (app);
+    g_free (url);
   }
 }
 
@@ -676,7 +681,7 @@ dbus_component_is_first_instance (GObject *object)
 
 void
 dbus_component_call_address (GObject *object, 
-			     gchar *address)
+			     const gchar *address)
 {
   DBusMessage *message = NULL;
   DBusComponent *self = NULL;
