@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2003 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
  *                         endpoint.h  -  description
  *                         --------------------------
  *   begin                : Sat Dec 23 2000
- *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : This file contains the Endpoint class.
  *
  */
@@ -39,24 +39,43 @@
 #ifndef _ENDPOINT_H_
 #define _ENDPOINT_H_
 
+
 #include "../config.h"
 
 #include "common.h"
-#include "gdkvideoio.h"
-#include "videograbber.h"
-#include "lid.h"
+
+#ifdef HAS_HOWL
+#include "zeroconf_publisher.h"
+#endif
 
 #ifdef HAS_IXJ
 #include <ixjlid.h>
 #endif
 
- 
+
+#include "gdkvideoio.h"
+#include "videograbber.h"
+#include "stunclient.h"
+
+class GMILSClient;
+class GMLid;
+class GMH323Gatekeeper;
+
+
+/**
+ * COMMON NOTICE: The Endpoint must be initialized with Init after its
+ * creation.
+ */
+
 class GMH323EndPoint : public H323EndPoint
 {
   PCLASSINFO(GMH323EndPoint, H323EndPoint);
 
   
  public:
+
+  enum CallingState {Standby, Calling, Connected, Called};
+
   
   /* DESCRIPTION  :  The constructor.
    * BEHAVIOR     :  Creates the local endpoint and initialises the variables
@@ -72,6 +91,14 @@ class GMH323EndPoint : public H323EndPoint
   ~GMH323EndPoint ();
 
 
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Init the endpoint internal values and the various
+   *                 components.
+   * PRE          :  /
+   */
+  void Init ();
+
+  
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Makes a call to the given address, and fills in the
    *                 call taken. It returns a locked pointer to the connection
@@ -82,13 +109,14 @@ class GMH323EndPoint : public H323EndPoint
 
   
   /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Updates some of the internal values of the endpoint such 
-   *                 as the local username, the capabilities, the tunneling,
-   *                 the fast start, the audio devices to use, the video 
-   *                 device to use, ...
+   * BEHAVIOR     :  Update the internal audio and video devices for playing
+   *                 and recording following the config database content.
+   *                 If a Quicknet card is used, it will be opened, and if
+   *                 a video grabber is used in preview mode, it will also
+   *"                be opened.
    * PRE          :  /
    */
-  void UpdateConfig ();
+  void UpdateDevices ();
 
 
   /* DESCRIPTION  :  /
@@ -97,15 +125,14 @@ class GMH323EndPoint : public H323EndPoint
    *                 grabbing after its creation. If TRUE,
    *                 then the opening is done sync.
    */  
-  void CreateVideoGrabber (BOOL = true, BOOL = false);
+  GMVideoGrabber *CreateVideoGrabber (BOOL = TRUE, BOOL = FALSE);
 
 
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Removes the current video grabber, if any.
-   * PRE          :  If TRUE, then wait until all video grabbers
-   *                 are removed before returning.
+   * PRE          :  /
    */  
-  void RemoveVideoGrabber (BOOL = false);
+  void RemoveVideoGrabber ();
 
 
   /* DESCRIPTION  :  /
@@ -155,7 +182,7 @@ class GMH323EndPoint : public H323EndPoint
 
 
   /* DESCRIPTION  :  This callback is called when a call is forwarded.
-   * BEHAVIOR     :  Outputs a message in the history and statusbar.
+   * BEHAVIOR     :  Outputs a message in the log and statusbar.
    * PRE          :  /
    */
   virtual BOOL OnConnectionForwarded (H323Connection &,
@@ -244,6 +271,21 @@ class GMH323EndPoint : public H323EndPoint
    */
   void RemoveAllCapabilities (void);
 
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Add all possible capabilities for the endpoint.
+   * PRE          :  /
+   */
+  void AddAllCapabilities (void);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Returns the list of audio capabilities supported by
+   * 		     the endpoint.
+   * PRE          :  /
+   */
+  OpalMediaFormat::List GMH323EndPoint::GetAvailableAudioCapabilities ();
+  
   
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Add audio capabilities following the user config.
@@ -253,12 +295,10 @@ class GMH323EndPoint : public H323EndPoint
 
   
   /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Add video capabilities, with QCIF as first video
-   *                 capability if the parameter is 0, else CIF will be the
-   *                 first video capability.
+   * BEHAVIOR     :  Add video capabilities.
    * PRE          :  /
    */
-  void AddVideoCapabilities (int);
+  void AddVideoCapabilities ();
 
 
   /* DESCRIPTION  :  /
@@ -279,22 +319,24 @@ class GMH323EndPoint : public H323EndPoint
   
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Set the current calling state :
-   *                   0 : not in a call
-   *                   1 : calling somebody
-   *                   2 : currently in a call 
+   *                   { Standby,
+   *                     Calling,
+   *                     Connected,
+   *                     Called }
    * PRE          :  /
    */
-  void SetCallingState (int);
+  void SetCallingState (CallingState);
 
 
   /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Get the current calling state :
-   *                   0 : not in a call
-   *                   1 : calling somebody
-   *                   2 : currently in a call 
+   * BEHAVIOR     :  Returns the current calling state :
+   *                   { Standby,
+   *                     Calling,
+   *                     Connected,
+   *                     Called }
    * PRE          :  /
    */
-  int GetCallingState (void);
+  CallingState GetCallingState (void);
 
 
   /* Overrides from H323Endpoint */
@@ -310,7 +352,7 @@ class GMH323EndPoint : public H323EndPoint
    *                 even if the endpoint is listening on many interfaces
    * PRE          :  /
    */
-  gchar *GetCurrentIP (void);
+  PString GetCurrentIP (void);
 
 
   /* DESCRIPTION  :  /
@@ -332,36 +374,15 @@ class GMH323EndPoint : public H323EndPoint
 
 
   /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Stops the current audio tester if any.
-   *                 a conference.
+   * BEHAVIOR     :  Stops the current audio tester if any for the given
+   *                 audio manager, player and recorder.
    * PRE          :  /
    */
-  void StartAudioTester ();
+  void StartAudioTester (gchar *,
+			 gchar *,
+			 gchar *);
 
 
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the current connection or 
-   *                 NULL if there is no one.
-   * PRE          :  /
-   */
-  H323Connection *GetCurrentConnection (void);
-
-
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the number of video channels in use.
-   *                 0 if there is no one, or if we are not in a call.
-   * PRE          :  /
-   */
-  int GetVideoChannelsNumber (void);
-
-
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Set the current connection to the parameter.
-   * PRE          :  A valid pointer to the current connection.
-   */
-  void SetCurrentConnection (H323Connection *);
-
-  
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Set the current call token.
    * PRE          :  A valid PString for a call token.
@@ -385,27 +406,105 @@ class GMH323EndPoint : public H323EndPoint
 
 
   /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Register to the gatekeeper given in the options, 
-   *                 if any.
+   * BEHAVIOR     :  Register to (or unregister from) the gatekeeper 
+   *                 given in the options, if any. Starts or stop the force
+   *                 renewal timer.
    * PRE          :  /
    */
   void GatekeeperRegister (void);
 
-
+  
   /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Return the current ILS/LDAP client thread.
+   * BEHAVIOR     :  Sets the given Stun Server as default STUN server for
+   * 		     calls.
    * PRE          :  /
    */
-  PThread* GetILSClientThread ();
+  void SetSTUNServer (void);
+
+
+#ifdef HAS_HOWL
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Update the information published by zeroconf.
+   * PRE          :  /
+   */
+  void ZeroconfUpdate (void);
+#endif
+
+  
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Register to (or unregister from) the ILS server
+   *                 given in the options, if any.
+   * PRE          :  /
+   */
+  void ILSRegister ();
 
 
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Set the local user name following the firstname and last 
-   *                 name stored by gconf, set the gatekeeper alias.
+   *                 name stored by the conf, set the gatekeeper alias.
    * PRE          :  /
    */
   void SetUserNameAndAlias ();
 
+    
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Update the RTP, TCP, UDP ports from the config database.
+   * PRE          :  /
+   */
+  void SetPorts ();
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Update the audio device volume (playing then recording). 
+   * PRE          :  /
+   */
+  BOOL SetDeviceVolume (PSoundChannel *, BOOL, unsigned int);
+
+  
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Returns the audio device volume (playing then recording). 
+   * PRE          :  /
+   */
+  BOOL GetDeviceVolume (PSoundChannel *, BOOL, unsigned int &);
+  
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  TRUE if the video should automatically be transmitted
+   *                 when a call begins.
+   * PRE          :  /
+   */
+  void SetAutoStartTransmitVideo (BOOL a) {autoStartTransmitVideo = a;}
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  TRUE if the video should automatically be received
+   *                 when a call begins.
+   * PRE          :  /
+   */
+  void SetAutoStartReceiveVideo (BOOL a) {autoStartReceiveVideo = a;}
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Automatically starts transmitting (BOOL = FALSE) or
+   *                 receiving (BOOL = TRUE) with the given
+   *                 capability, for the given session RTP id and returns
+   *                 TRUE if it was successful. Do that only if there is
+   *                 a connection.
+   * PRE          :  /
+   */
+  BOOL StartLogicalChannel (const PString &, unsigned int, BOOL);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Automatically stop transmitting (BOOL = FALSE) or
+   *                 receiving (BOOL = TRUE) for the given RTP session id
+   *                 and returns TRUE if it was successful.
+   *                 Do that only if there is a connection.
+   * PRE          :  /
+   */
+  BOOL StopLogicalChannel (unsigned int, BOOL);
+
 
 #ifdef HAS_IXJ
   /* DESCRIPTION  :  /
@@ -426,15 +525,117 @@ class GMH323EndPoint : public H323EndPoint
 
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Create a new Lid.
-   * PRE          :  /
+   * PRE          :  Non-empty device name.
    */
-  void CreateLid ();
-
+  GMLid *CreateLid (PString);
 #endif
 
 
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Sends the given text to the other end
+   * PRE          :  /
+   */
+  void SendTextMessage (PString callToken, PString message);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Checks if the call is on hold
+   * PRE          :  /
+   */
+  BOOL IsCallOnHold (PString callToken);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Sets the call on hold or retrieve it and returns
+   * 		     FALSE if it fails, TRUE if it worked.
+   * PRE          :  /
+   */
+  BOOL SetCallOnHold (PString callToken, 
+		      gboolean state);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Send DTMF to the remote user if any.
+   * PRE          :  /
+   */
+  void SendDTMF (PString,
+		 PString);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Checks if the call has an audio channel
+   * PRE          :  Non-empty call token.
+   */
+  gboolean IsCallWithAudio (PString callToken);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Checks if the call has a video channel
+   * PRE          :  Non-empty call token.
+   */
+  gboolean IsCallWithVideo (PString callToken);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Checks if the call's audio channel is paused
+   * PRE          :  Non-empty call token.
+   */
+  gboolean IsCallAudioPaused (PString callToken);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Checks if the call's video channel is paused
+   * PRE          :  Non-empty call token.
+   */
+  gboolean IsCallVideoPaused (PString callToken);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Sets the call's audio channel on pause, or retrieve it
+   * PRE          :  Non-empty call token.
+   */
+  BOOL SetCallAudioPause (PString callToken, 
+			  BOOL state);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Sets the call's video channel on pause, or retrieve it
+   * PRE          :  Non-empty call token.
+   */
+  BOOL SetCallVideoPause (PString callToken, 
+			  BOOL state);
+  
+  
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Reset the missed calls number.
+   * PRE          :  /
+   */
+  void ResetMissedCallsNumber ();
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Return the missed calls number.
+   * PRE          :  /
+   */
+  int GetMissedCallsNumber ();
+  
+  
+  /* DESCRIPTION  : /
+   * BEHAVIOR     : Adds the observer to the list of GObject interested in
+   *                the endpoint's signals.
+   * PRE          : Non-empty GObject
+   */
+  void AddObserver (GObject *observer);
+
  protected:
 
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Returns the last called call token.
+   * PRE          :  /
+   */
+  PString GetLastCallAddress ();
+
+  
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Set the current transfer call token.
    * PRE          :  A valid PString for a call token.
@@ -456,7 +657,84 @@ class GMH323EndPoint : public H323EndPoint
    */
   void TransferCallWait ();
 
+  
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Set (BOOL = TRUE) or get (BOOL = FALSE) the
+   *                 audio playing and recording volumes for the
+   *                 audio device.
+   * PRE          :  /
+   */
+  BOOL DeviceVolume (PSoundChannel *, BOOL, BOOL, unsigned int &);
 
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Check if the listening port is accessible from the
+   *                 outside and returns the test result given by seconix.com.
+   * PRE          :  /
+   */
+  PString CheckTCPPorts ();
+
+  
+  /* DESCRIPTION  :  Notifier called after 30 seconds of no media.
+   * BEHAVIOR     :  Clear the calls.
+   * PRE          :  /
+   */
+  PDECLARE_NOTIFIER(PTimer, GMH323EndPoint, OnNoIncomingMediaTimeout);
+
+  
+  /* DESCRIPTION  :  Notifier called periodically to update details on ILS.
+   * BEHAVIOR     :  Register, unregister the user from ILS or udpate his
+   *                 personal data using the GMILSClient (XDAP).
+   * PRE          :  /
+   */
+  PDECLARE_NOTIFIER(PTimer, GMH323EndPoint, OnILSTimeout);
+
+
+  /* DESCRIPTION  :  Notifier called periodically during calls.
+   * BEHAVIOR     :  Refresh the statistics window of the Control Panel
+   *                 if it is currently shown.
+   * PRE          :  /
+   */
+  PDECLARE_NOTIFIER(PTimer, GMH323EndPoint, OnRTPTimeout);
+
+
+  /* DESCRIPTION  :  Notifier called periodically to update the gateway IP.
+   * BEHAVIOR     :  Update the gateway IP to use for the IP translation
+   *                 if IP Checking is enabled in the config database.
+   * PRE          :  /
+   */
+  PDECLARE_NOTIFIER(PTimer, GMH323EndPoint, OnGatewayIPTimeout);
+
+  
+  /* DESCRIPTION  :  Notifier called when an incoming call
+   *                 has not been answered after 15 seconds.
+   * BEHAVIOR     :  Reject the call, or forward if forward on no answer is
+   *                 enabled in the config database.
+   * PRE          :  /
+   */
+  PDECLARE_NOTIFIER(PTimer, GMH323EndPoint, OnNoAnswerTimeout);
+
+
+  /* DESCRIPTION  :  Notifier called every second while waiting for an answer
+   *                 for an incoming call.
+   * BEHAVIOR     :  Display an animation in the docklet and play a ring
+   *                 sound.
+   * PRE          :  /
+   */
+  PDECLARE_NOTIFIER(PTimer, GMH323EndPoint, OnCallPending);
+
+
+  /* DESCRIPTION  :  Notifier called every 2 seconds while waiting for an
+   *                 answer for an outging call.
+   * BEHAVIOR     :  Display an animation in the main winow and play a ring
+   *                 sound.
+   * PRE          :  /
+   */
+  PDECLARE_NOTIFIER(PTimer, GMH323EndPoint, OnOutgoingCall);
+
+  GtkWidget *audio_transmission_popup;
+  GtkWidget *audio_reception_popup;
+  
   PString called_address;
   PString current_call_token;
   PString transfer_call_token;
@@ -464,46 +742,65 @@ class GMH323EndPoint : public H323EndPoint
   H323Connection *current_connection;  
   H323ListenerTCP *listener;  
 
-  int calling_state; 
-  int docklet_timeout; 
-  int sound_timeout; 
-  int display_config; 
+  CallingState calling_state; 
+  GObject *dispatcher;
 
-  GDKVideoOutputDevice *transmitted_video_device; 
-  GDKVideoOutputDevice *received_video_device; 
+  PTimer ILSTimer;
+  PTimer RTPTimer;
+  PTimer GatewayIPTimer;
+  PTimer NoAnswerTimer;
+  PTimer CallPendingTimer;
+  PTimer OutgoingCallTimer;
+    
+  BOOL ils_registered;
 
-  PSoundChannel *player_channel;
-  PSoundChannel *recorder_channel;
 
-  GmWindow *gw; 
-  GmLdapWindow *lw;
-  GmTextChat *chat;
-  GConfClient *client;
+  int missed_calls;
 
+
+  /* detection of no incoming data */
+  int last_audio_octets_received;
+  int last_video_octets_received;
+  int last_audio_octets_transmitted;
+  int last_video_octets_transmitted;
+  PTimer NoIncomingMediaTimer;
+
+
+  /* The encoding video grabber */
   GMVideoGrabber *video_grabber;
-  PThread *ils_client;
+
+  GMH323Gatekeeper *gk;
+  GMStunClient *sc;
+  
+  GMILSClient *ils_client;
   PThread *audio_tester;
 
-  /* Mutexes for the different variables to make
-     sure there is not 2 threads accessing them
-     at the same time */
+
+  /* Various mutexes to ensure thread safeness around internal
+     variables */
   PMutex vg_access_mutex;
   PMutex ils_access_mutex;
-  PMutex cc_access_mutex;
-  PMutex ch_access_mutex;
   PMutex cs_access_mutex;
   PMutex ct_access_mutex;
   PMutex tct_access_mutex;
   PMutex lid_access_mutex;
-  
-  PIntCondMutex *vg;
+  PMutex at_access_mutex;
+  PMutex lca_access_mutex;
+  PMutex mc_access_mutex;
 
-  int opened_audio_channels;
-  int opened_video_channels;
+  PMutex sound_event_mutex;
+  
+  PMutex audio_channel_mutex;
+  PMutex video_channel_mutex;
 
 #ifdef HAS_IXJ
   GMLid *lid;
 #endif
+
+#ifdef HAS_HOWL
+  GMZeroconfPublisher *zcp;
+  PMutex zcp_access_mutex;
+#endif
 };
 
 #endif
