@@ -63,7 +63,7 @@ class GMILSClient : public PObject
   /* DESCRIPTION  :  The constructor.
    * BEHAVIOR     :  Creates the ILS registering thread and starts it. It will
    *                 automatically register to the ILS server given in the
-   *                 gconf preferences if the user chose to register.
+   *                 config preferences if the user chose to register.
    * PRE          :  /
    */
   GMILSClient ();
@@ -79,7 +79,7 @@ class GMILSClient : public PObject
   /* DESCRIPTION  :  /
    * BEHAVIOR     :  Set the flag to register to 1. When all operations will
    *                 be terminated, the thread will try to register to the
-   *                 ILS server. Options are stored in the GConf database.
+   *                 ILS server. Options are stored in the config database.
    * PRE          :  /
    */
   void Register ();
@@ -98,7 +98,7 @@ class GMILSClient : public PObject
    * BEHAVIOR     :  Set the flag to modify to 1. When all operations will
    *                 be terminated, the thread will try to update the
    *                 current registered options on the ILS server. Options
-   *                 are stored in the GConf database.
+   *                 are stored in the config database.
    * PRE          :  /
    */
   void Modify ();
@@ -133,52 +133,10 @@ protected:
   void ILSOperation (Operation);
 
 
-  GmWindow *gw;
-  GmLdapWindow *lw;
-
   int operation;
 
   PTime starttime;
   LDAP *ldap_connection;
   PMutex quit_mutex;
 };
-
-
-/* The Browser class */
-class GMILSBrowser : public PThread
-{
-  PCLASSINFO(GMILSBrowser, PThread);
-
- public:
-  
-  /* DESCRIPTION  :  The constructor.
-   * BEHAVIOR     :  Initialise the parameters.
-   * PRE          :  The structure of widgets for that page.
-   *                 The LDAP Server to browse (non-empty string).
-   *                 The search filter, can be NULL (no filter).
-   */
-  GMILSBrowser (GmLdapWindowPage *, gchar *, gchar * = NULL);
-
-
-  /* DESCRIPTION  :  The destructor.
-   * BEHAVIOR     :  /
-   * PRE          :  /
-   */
-  ~GMILSBrowser ();
-
-
- protected:
-
-
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Browses the server and fills in the list.
-   * PRE          :  /
-   */
-  void Main ();
-
-  GmLdapWindow *lw;
-  GmLdapWindowPage *lwp;
-  gchar *ldap_server;
-  gchar *search_filter;
-};
 #endif
