@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2003 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
  *                         sound_handling.cpp  -  description
  *                         ----------------------------------
  *   begin                : Thu Nov 22 2001
- *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : This file contains sound handling functions.
  *
  */
@@ -43,10 +43,11 @@
 #include "endpoint.h"
 #include "lid.h"
 #include "misc.h"
+#include "druid.h"
 
 #include "gtklevelmeter.h"
 #include "dialog.h"
-#include "gconf_widgets_extensions.h"
+#include "gm_conf.h"
 
 
 #ifdef HAS_IXJ
@@ -68,21 +69,18 @@
 #include <ptclib/pwavfile.h>
 #include <cmath>
 
-
 static void dialog_response_cb (GtkWidget *, gint, gpointer);
-  
-extern GtkWidget *gm;
 
 
 /* The GTK callbacks */
-void dialog_response_cb (GtkWidget *w, gint, gpointer data)
+static void dialog_response_cb (GtkWidget *w, 
+				gint, 
+				gpointer data)
 {
-  GmDruidWindow *dw = NULL;
+  g_return_if_fail (data);
 
-  dw = GnomeMeeting::Process ()->GetDruidWindow ();
 
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dw->audio_test_button),
-				false);
+  gm_druid_window_set_test_buttons_sensitivity (GTK_WIDGET (data), FALSE);
 
   gtk_widget_hide (w);
 }
@@ -93,18 +91,16 @@ void
 gnomemeeting_sound_daemons_suspend (void)
 {
 #ifndef WIN32
-#if defined(HAS_ESD)
-  return;
-#else
   int esd_client = 0;
   
   /* Put ESD into standby mode */
   esd_client = esd_open_sound (NULL);
 
-  esd_standby (esd_client);
-      
-  esd_close (esd_client);
-#endif
+  if (esd_client >= 0) {
+    
+    esd_standby (esd_client);
+    esd_close (esd_client);
+  }
 #endif
 }
 
@@ -113,18 +109,16 @@ void
 gnomemeeting_sound_daemons_resume (void)
 {
 #ifndef WIN32
-#if defined(HAS_ESD)
-  return;
-#else
   int esd_client = 0;
 
   /* Put ESD into normal mode */
   esd_client = esd_open_sound (NULL);
 
-  esd_resume (esd_client);
-
-  esd_close (esd_client);
-#endif
+  if (esd_client >= 0) {
+    
+    esd_resume (esd_client);
+    esd_close (esd_client);
+  }
 #endif
 }
 
@@ -169,7 +163,9 @@ GMSoundEvent::GMSoundEvent (PString ev)
 {
   event = ev;
 
+  gnomemeeting_sound_daemons_suspend ();
   Main ();
+  gnomemeeting_sound_daemons_resume ();
 }
 
 
@@ -185,21 +181,24 @@ void GMSoundEvent::Main ()
   PBYTEArray buffer;  
 
   PString psound_file;
-  PString enable_event_gconf_key;
-  PString event_gconf_key;
+  PString enable_event_conf_key;
+  PString event_conf_key;
   
-  plugin = gconf_get_string (AUDIO_DEVICES_KEY "plugin");
-  device = gconf_get_string (AUDIO_DEVICES_KEY "output_device");
+  plugin = gm_conf_get_string (AUDIO_DEVICES_KEY "plugin");
+  if (event == "incoming_call_sound")
+    device = gm_conf_get_string (SOUND_EVENTS_KEY "output_device");
+  else
+    device = gm_conf_get_string (AUDIO_DEVICES_KEY "output_device");
 
-  enable_event_gconf_key = PString (SOUND_EVENTS_KEY) + "enable_" + event;
+  enable_event_conf_key = PString (SOUND_EVENTS_KEY) + "enable_" + event;
   if (event.Find ("/") == P_MAX_INDEX) {
     
-    event_gconf_key = PString (SOUND_EVENTS_KEY) + event;
-    sound_file = gconf_get_string ((gchar *) (const char *) event_gconf_key);
+    event_conf_key = PString (SOUND_EVENTS_KEY) + event;
+    sound_file = gm_conf_get_string ((gchar *) (const char *) event_conf_key);
   }
   
   if (!sound_file ||
-      gconf_get_bool ((gchar *) (const char *) enable_event_gconf_key)) {
+      gm_conf_get_bool ((gchar *) (const char *) enable_event_conf_key)) {
 
     if (!sound_file)    
       sound_file = g_strdup ((const char *) event);
@@ -268,8 +267,10 @@ GMAudioRP::~GMAudioRP ()
 
 void GMAudioRP::Main ()
 {
+  GtkWidget *druid_window = NULL;
+  
   PSoundChannel *channel = NULL;
-  GmWindow *gw = NULL;
+
 
 #ifdef HAS_IXJ
   /* We only have one GMLid thread, so use class variables instead
@@ -298,7 +299,7 @@ void GMAudioRP::Main ()
   PWaitAndSignal m(quit_mutex);
   thread_sync_point.Signal ();
 
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  druid_window = GnomeMeeting::Process ()->GetDruidWindow ();
   ep = GnomeMeeting::Process ()->Endpoint ();
     
   buffer = (char *) malloc (640);
@@ -339,9 +340,9 @@ void GMAudioRP::Main ()
 
     gdk_threads_enter ();  
     if (is_encoding)
-      gnomemeeting_error_dialog (GTK_WINDOW (gw->druid_window), _("Failed to open the device"), _("Impossible to open the selected audio device (%s) for recording. Please check your audio setup, the permissions and that the device is not busy."), (const char *) device_name);
+      gnomemeeting_error_dialog (GTK_WINDOW (druid_window), _("Failed to open the device"), _("Impossible to open the selected audio device (%s) for recording. Please check your audio setup, the permissions and that the device is not busy."), (const char *) device_name);
     else
-      gnomemeeting_error_dialog (GTK_WINDOW (gw->druid_window), _("Failed to open the device"), _("Impossible to open the selected audio device (%s) for playing. Please check your audio setup, the permissions and that the device is not busy."), (const char *) device_name);
+      gnomemeeting_error_dialog (GTK_WINDOW (druid_window), _("Failed to open the device"), _("Impossible to open the selected audio device (%s) for playing. Please check your audio setup, the permissions and that the device is not busy."), (const char *) device_name);
     gdk_threads_leave ();  
   }
   else {
@@ -361,7 +362,6 @@ void GMAudioRP::Main ()
 	l->SetWriteFormat (0, OpalPCM16);
 
 	l->StopTone (0);
-	cout << l->IsTonePlaying (0) << endl << flush;
       }
     }
 #endif
@@ -381,7 +381,7 @@ void GMAudioRP::Main ()
 		&& !l->ReadFrame (0, (void *) buffer, i)))  {
       
 	  gdk_threads_enter ();  
-	  gnomemeeting_error_dialog (GTK_WINDOW (gw->druid_window), _("Cannot use the audio device"), _("The selected audio device (%s) was successfully opened but it is impossible to read data from this device. Please check your audio setup."), (const char*) device_name);
+	  gnomemeeting_error_dialog (GTK_WINDOW (druid_window), _("Cannot use the audio device"), _("The selected audio device (%s) was successfully opened but it is impossible to read data from this device. Please check your audio setup."), (const char*) device_name);
 	  gdk_threads_leave ();  
 
 	  stop = TRUE;
@@ -395,7 +395,7 @@ void GMAudioRP::Main ()
 	    gdk_threads_enter ();
 	    gtk_label_set_markup (GTK_LABEL (tester->test_label), msg);
 	    if (nbr_opened_channels == 2)
-	      gtk_widget_show_all (GTK_WIDGET (tester->test_dialog));
+	      gnomemeeting_threads_dialog_show (GTK_WIDGET (tester->test_dialog));
 	    gdk_threads_leave ();
 	    g_free (msg);
 
@@ -437,7 +437,7 @@ void GMAudioRP::Main ()
 	    gdk_threads_enter ();
 	    gtk_label_set_markup (GTK_LABEL (tester->test_label), msg);
 	    if (nbr_opened_channels == 2)
-	      gtk_widget_show_all (GTK_WIDGET (tester->test_dialog));
+	      gnomemeeting_threads_dialog_show (GTK_WIDGET (tester->test_dialog));
 	    gdk_threads_leave ();
 	    g_free (msg);
 
@@ -457,7 +457,7 @@ void GMAudioRP::Main ()
 		  && !l->WriteFrame (0, (const void *) buffer, 640, i)))  {
       
 	    gdk_threads_enter ();  
-	    gnomemeeting_error_dialog (GTK_WINDOW (gw->druid_window), _("Cannot use the audio device"), _("The selected audio device (%s) was successfully opened but it is impossible to write data to this device. Please check your audio setup."), (const char*) device_name);
+	    gnomemeeting_error_dialog (GTK_WINDOW (druid_window), _("Cannot use the audio device"), _("The selected audio device (%s) was successfully opened but it is impossible to write data to this device. Please check your audio setup."), (const char*) device_name);
 	    gdk_threads_leave ();  
 
 	    stop = TRUE;
@@ -519,10 +519,8 @@ GMAudioRP::GetAverageSignalLevel (const short *buffer, int size)
 				gchar *r)
   :PThread (1000, NoAutoDeleteThread)
 {
-#ifndef DISABLE_GNOME
   stop = FALSE;
 
-  gw = GnomeMeeting::Process ()->GetMainWindow ();
   test_dialog = NULL;
   test_label = NULL;
   
@@ -532,7 +530,6 @@ GMAudioRP::GetAverageSignalLevel (const short *buffer, int size)
     audio_player = PString (p);
   if (r)
     audio_recorder = PString (r);
-#endif
   
   this->Resume ();
   thread_sync_point.Wait ();
@@ -541,23 +538,22 @@ GMAudioRP::GetAverageSignalLevel (const short *buffer, int size)
 
 GMAudioTester::~GMAudioTester ()
 {
-#ifndef DISABLE_GNOME
   stop = 1;
   PWaitAndSignal m(quit_mutex);
-#endif
 }
 
 
 void GMAudioTester::Main ()
 {
-#ifndef DISABLE_GNOME
+  GtkWidget *druid_window = NULL;
+  
   GMAudioRP *player = NULL;
   GMAudioRP *recorder = NULL;
-  GmDruidWindow *dw = NULL;
 
   gchar *msg = NULL;
 
-  dw = GnomeMeeting::Process ()->GetDruidWindow ();
+  druid_window = GnomeMeeting::Process ()->GetDruidWindow ();
+
 
   PWaitAndSignal m(quit_mutex);
   thread_sync_point.Signal ();
@@ -578,8 +574,8 @@ void GMAudioTester::Main ()
   gdk_threads_enter ();
   test_dialog =
     gtk_dialog_new_with_buttons ("Audio test running",
-				 GTK_WINDOW (gw->druid_window),
-				 (enum GtkDialogFlags) (GTK_DIALOG_MODAL),
+				 GTK_WINDOW (druid_window),
+				 (GtkDialogFlags) (GTK_DIALOG_MODAL),
 				 GTK_STOCK_OK,
 				 GTK_RESPONSE_ACCEPT,
 				 NULL);
@@ -605,10 +601,12 @@ void GMAudioTester::Main ()
   g_signal_connect (G_OBJECT (test_dialog), "delete-event",
 		    G_CALLBACK (gtk_widget_hide_on_delete), NULL);
   g_signal_connect (G_OBJECT (test_dialog), "response",
-		    G_CALLBACK (dialog_response_cb), NULL);
+		    G_CALLBACK (dialog_response_cb), 
+		    druid_window);
 
   gtk_window_set_transient_for (GTK_WINDOW (test_dialog),
-				GTK_WINDOW (gw->druid_window));
+				GTK_WINDOW (druid_window));
+  gtk_widget_show_all (GTK_DIALOG (test_dialog)->vbox);
   gdk_threads_leave ();
 
   recorder = new GMAudioRP (this, audio_manager, audio_recorder, TRUE);
@@ -631,5 +629,4 @@ void GMAudioTester::Main ()
   gnomemeeting_sound_daemons_resume ();
   
   free (buffer_ring);
-#endif
 }
